package core.services;

import core.api.models.timemanager.tasks.TaskFieldInfo;
import core.api.models.timemanager.tasks.TaskScope;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.Task;
import core.entities.timemanager.Taskgroup;
import core.repositories.timemanager.TaskRepository;
import core.repositories.timemanager.TaskgroupRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;

@Service
public class TaskService {

    private final TaskRepository taskRepository;
    private final TaskScheduleService taskScheduleService;
    private final TaskgroupRepository taskgroupRepository;


    public TaskService(@Autowired TaskRepository taskRepository,
                       @Autowired TaskScheduleService taskScheduleService,
                       TaskgroupRepository taskgroupRepository) {
        this.taskRepository = taskRepository;
        this.taskScheduleService = taskScheduleService;
        this.taskgroupRepository = taskgroupRepository;
    }

    public ServiceResult<Task> createTask(Taskgroup taskgroup, TaskFieldInfo taskFieldInfo) {
        if(existTaskByName(taskgroup.getTasks(), taskFieldInfo.getTaskName())) {
            return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
        }

        //Check for invalid date (deadline before start
        if(taskFieldInfo.getStartDate() != null && taskFieldInfo.getDeadline() != null &&
                taskFieldInfo.getDeadline().isBefore(taskFieldInfo.getStartDate())) {
            return new ServiceResult<>(ServiceExitCode.INVALID_PARAMETER);
        }

        Task task = new Task(taskgroup, taskFieldInfo);
        taskgroup.getTasks().add(task);
        taskRepository.save(task);

        return new ServiceResult<>(task);
    }

    private boolean existTaskByName(Collection<Task> tasks, String name) {
        for(Task task : tasks) {
            if(task.getTaskName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    public PermissionResult<Task> getTaskPermissions(long taskID, String name) {
        Optional<Task> task = taskRepository.findById(taskID);
        return task.map(value -> new PermissionResult<>(value, value.getTaskgroup().getUser().getUsername().equals(name))).orElseGet(() ->
                new PermissionResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<Task> editTask(Task task, TaskFieldInfo taskFieldInfo) {
        if(!task.getTaskName().equals(taskFieldInfo.getTaskName())) {
            //taskname will be updated
            if(existTaskByName(task.getTaskgroup().getTasks(), taskFieldInfo.getTaskName())) {
                return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
            } else {
                task.setTaskName(taskFieldInfo.getTaskName());
            }
        }

        //Check for invalid date (deadline before start
        if(taskFieldInfo.getStartDate() != null && taskFieldInfo.getDeadline() != null &&
                taskFieldInfo.getDeadline().isBefore(taskFieldInfo.getStartDate())) {
            return new ServiceResult<>(ServiceExitCode.INVALID_PARAMETER);
        }

        task.setEta(taskFieldInfo.getEta());
        task.setStartDate(taskFieldInfo.getStartDate());
        task.setDeadline(taskFieldInfo.getDeadline());
        taskRepository.save(task);
        return new ServiceResult<>(task);
    }

    public void deleteTask(Task task) {
        //taskScheduleService.deleteScheduleByTask(task);
        System.err.println(task.getTaskID());
        task.getTaskgroup().getTasks().remove(task);
        taskgroupRepository.save(task.getTaskgroup());
        task.setTaskgroup(null);
        taskRepository.save(task);
        taskRepository.delete(task);
    }

    public void clearTasks(Taskgroup taskgroup) {
        taskRepository.deleteAllByTaskgroup(taskgroup);
    }

    public void finishTask(Task task) {
        //Mark task as finished and remove all unstarted schedules as well as finish all started schedules
        task.finish();
        taskRepository.save(task);


        List<AbstractSchedule> removedBasicTaskSchedules = new LinkedList<>();
        for(AbstractSchedule basicTaskSchedule : task.getBasicTaskSchedules()) {
            if(basicTaskSchedule.getStartTime() == null) {
                removedBasicTaskSchedules.add(basicTaskSchedule);
            } else if(basicTaskSchedule.getStopTime() == null) {
                ServiceResult<?> result = taskScheduleService.stopSchedule(basicTaskSchedule, true);
                System.out.println(result);
            }
        }

        task.getBasicTaskSchedules().removeAll(removedBasicTaskSchedules);
        taskRepository.save(task);
        taskScheduleService.deleteSchedules(removedBasicTaskSchedules);
    }

    public List<Task> loadAllTasks(String username, TaskScope scope) {
        switch (scope) {
            case UNFINISHED, FINISHED -> {
                return taskRepository.findAllByUser(username, scope.equals(TaskScope.FINISHED));
            }
            case OVERDUE -> {
                return taskRepository.findAllOverdue(username, LocalDate.now());
            }
            case UPCOMING -> {
                return taskRepository.findAllUpcoming(username, LocalDate.now());
            }
            case ACTIVE -> {
                return taskRepository.findAllActive(username, LocalDate.now());
            }
            default -> {
                return new ArrayList<>();
            }
        }

    }
}
