package core.taskgroups;

import core.api.models.timemanager.taskgroup.TaskgroupFieldInfo;
import core.entities.User;
import core.entities.timemanager.Task;
import core.entities.timemanager.Taskgroup;
import core.repositories.UserRepository;
import core.repositories.timemanager.TaskgroupRepository;
import core.services.PermissionResult;
import core.services.ServiceExitCode;
import core.services.ServiceResult;
import core.services.TaskgroupService;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.jdbc.SqlGroup;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.context.junit4.SpringRunner;

import javax.persistence.EntityManager;
import javax.transaction.Transactional;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest
@Transactional
public class TaskgroupServiceTest {

    @Autowired
    private TaskgroupRepository taskgroupRepository;
    @Autowired
    private TaskgroupService taskgroupService;
    @Autowired
    private EntityManager entityManager;

    private static final String username = "Testuser1";
    private static final String username2 = "Testuser2";
    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void getTaskgroupByIDAndUsername() {
        //Situation 1: correct taskgroup and username
        assertFalse(taskgroupRepository.findById(1L).isEmpty());

        PermissionResult<Taskgroup> permissionResult = taskgroupService.getTaskgroupByIDAndUsername(1L, username);
        assertThat(permissionResult).isNotNull();
        assertTrue(permissionResult.isHasPermissions());
        assertEquals(ServiceExitCode.OK, permissionResult.getExitCode());
        assertEquals(taskgroupRepository.findById(1L).get(), permissionResult.getResult());

        //Situation 2: invalid taskgroup
        PermissionResult<Taskgroup> invalid_group = taskgroupService.getTaskgroupByIDAndUsername(-1L, username);
        assertThat(invalid_group).isNotNull();
        assertEquals(ServiceExitCode.MISSING_ENTITY, invalid_group.getExitCode());
        assertFalse(invalid_group.isHasPermissions());

        //Situation 3: invalid user
        PermissionResult<Taskgroup> invalid_user = taskgroupService.getTaskgroupByIDAndUsername(1L, "Rotzbakke");
        assertThat(invalid_user).isNotNull();
        assertEquals(ServiceExitCode.OK, invalid_user.getExitCode());
        assertFalse(invalid_user.isHasPermissions());
    }

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void addTaskgroup() {
        TaskgroupFieldInfo taskgroupFieldInfo = new TaskgroupFieldInfo("Taskgroup", -1);
        //Situation 1: No such user
        assertThrows(NoSuchElementException.class, () -> taskgroupService.addTaskgroup(taskgroupFieldInfo, "Rotzbakke"));

        //Situation 2: Taskgroup already exists
        Taskgroup taskgroup = taskgroupRepository.findById(1L).get();
        ServiceResult<Taskgroup> creationResult = taskgroupService.addTaskgroup(new TaskgroupFieldInfo(taskgroup.getTaskgroupName(), -1), username);
        assertEquals(ServiceExitCode.ENTITY_ALREADY_EXIST, creationResult.getExitCode());
        assertThat(creationResult.getResult()).isNull();

        //Situation 3: Taskgroup does not exist, no parent
        ServiceResult<Taskgroup> creationResult_3 = taskgroupService.addTaskgroup(taskgroupFieldInfo, username);
        assertEquals(ServiceExitCode.OK, creationResult_3.getExitCode());
        assertTrue(taskgroupRepository.existsById(creationResult_3.getResult().getTaskgroupID()));

        //Situation 4: Taskgroup does not exist, with parent; invalid parent (no existent)
        ServiceResult<Taskgroup> creationResult_4 = taskgroupService.addTaskgroup(new TaskgroupFieldInfo("Situation 4", 100L), username);
        assertEquals(ServiceExitCode.MISSING_ENTITY, creationResult_4.getExitCode());

        //Situation 5: Taskgroup does not exist, parent exist
        ServiceResult<Taskgroup> creationResult_5 = taskgroupService.addTaskgroup(new TaskgroupFieldInfo("Situation 5", 2L), username);
        assertEquals(ServiceExitCode.OK, creationResult_5.getExitCode());
        assertTrue(taskgroupRepository.existsById(creationResult_5.getResult().getTaskgroupID()));

        //Situation 6: taskgroup exist on another user
        ServiceResult<Taskgroup> creationResult_6 = taskgroupService.addTaskgroup(new TaskgroupFieldInfo(taskgroup.getTaskgroupName(), -1), username2);
        assertEquals(ServiceExitCode.OK, creationResult_6.getExitCode());
        assertTrue(taskgroupRepository.existsById(creationResult_6.getResult().getTaskgroupID()));

    }

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void editTaskgroup() {
        Taskgroup taskgroup = taskgroupRepository.findById(1L).get();
        //Situation 1: Nothing changed
        ServiceExitCode exitCode_1 = taskgroupService.editTaskgroup(taskgroup, new TaskgroupFieldInfo(taskgroup.getTaskgroupName(),-1));
        assertEquals(ServiceExitCode.OK, exitCode_1);

        //Situation 2: Name is already taken
        ServiceExitCode exitCode_2 = taskgroupService.editTaskgroup(taskgroup, new TaskgroupFieldInfo("Taskgroup 1", -1));
        assertEquals(ServiceExitCode.ENTITY_ALREADY_EXIST, exitCode_2);

        //Situation 3: All fine
        ServiceExitCode exitCode_3 = taskgroupService.editTaskgroup(taskgroup, new TaskgroupFieldInfo("Situation 3", -1));
        assertEquals(ServiceExitCode.OK, exitCode_3);
    }

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void deleteTaskgroup() {
        taskgroupService.deleteTaskgroup(entityManager.find(Taskgroup.class, 1L));
        assertThat(entityManager.find(Taskgroup.class, 1L)).isNull();

        taskgroupService.deleteTaskgroup(entityManager.find(Taskgroup.class, 2L));
        assertThat(entityManager.find(Taskgroup.class, 2L)).isNull();
        assertThat(entityManager.find(Taskgroup.class, 3L)).isNull();
        for(long i=1; i<=14; i++) {
            assertThat(entityManager.find(Task.class, i)).isNull();
        }

        taskgroupService.deleteTaskgroup(entityManager.find(Taskgroup.class, 5L));
        assertThat(entityManager.find(Taskgroup.class, 5L)).isNull();
        assertThat(entityManager.find(Taskgroup.class, 6L)).isNull();
    }

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void getTopTaskgroupsByUser() {
        //Situation 1: User without taskgroups
        List<Taskgroup> taskgroups_user2 = taskgroupService.getTopTaskgroupsByUser(username2);
        assertEquals(0, taskgroups_user2.size());


        //Situation 2: user with taskgroups
        List<Taskgroup> taskgroups_user1 = taskgroupService.getTopTaskgroupsByUser(username);
        assertEquals(3, taskgroups_user1.size());
        assertTrue(taskgroups_user1.contains(entityManager.find(Taskgroup.class, 1L)));
        assertTrue(taskgroups_user1.contains(entityManager.find(Taskgroup.class, 2L)));
        assertTrue(taskgroups_user1.contains(entityManager.find(Taskgroup.class, 5L)));

        //Situation 3: No existent username
        assertEquals(0, taskgroupService.getTopTaskgroupsByUser("Rotzbakke").size());
    }
    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void deleteTaskgroupByUser() {
        User referenceUser1 = entityManager.find(User.class, 1L);
        User referenceUser2 = entityManager.find(User.class, 2L);

        taskgroupService.deleteTaskgroupByUser(referenceUser2);
        for(long i=1; i<=8; i++) {
            assertThat(entityManager.find(Taskgroup.class, i)).isNotNull();
        }

        taskgroupService.deleteTaskgroupByUser(referenceUser1);
        for(long i=1; i<=8; i++) {
            assertThat(entityManager.find(Taskgroup.class, i)).isNull();
        }
        for(long i=1; i<=14; i++) {
            assertThat(entityManager.find(Task.class, i)).isNull();
        }
    }

}
